@@ -28,11 +28,26 @@
 
 static GTK_COMPAT_ADJUSTMENT_TYPE compression_value;
 
-static void add_fresh_update_toggled_cb (GtkToggleButton *button, Add_dialog_data *data)
+static gboolean no_focus (GtkWidget *widget, GtkDirectionType direction, gpointer user_data)
 {
-	gboolean active = gtk_toggle_button_get_active (button);
+	return TRUE;
+}
+
+static void toggle_overwrite_update_freshen (GtkToggleButton *button, Add_dialog_data *data)
+{
+	gboolean active = gtk_toggle_button_get_active(button);
+
 	if (active)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->update), FALSE);
+	{
+		if (button != GTK_TOGGLE_BUTTON(data->overwrite))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->overwrite), FALSE);
+
+		if (button != GTK_TOGGLE_BUTTON(data->update))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->update), FALSE);
+
+		if (button != GTK_TOGGLE_BUTTON(data->freshen))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->freshen), FALSE);
+	}
 }
 
 static void password_toggled_cb (GtkToggleButton *button, Add_dialog_data *add_dialog)
@@ -46,22 +61,23 @@ static void password_toggled_cb (GtkToggleButton *button, Add_dialog_data *add_d
 		gtk_widget_set_sensitive (add_dialog->add_password_entry, FALSE);
 }
 
-static void add_update_fresh_toggled_cb (GtkToggleButton *button, Add_dialog_data *data)
+static void toggle_compression (GtkToggleButton *button, Add_dialog_data *add_dialog)
 {
-	if (data->freshen == NULL)
-		return;
-	gboolean active = gtk_toggle_button_get_active (button);
-	if (active)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->freshen), FALSE);
+	gboolean toggle_button_active = gtk_toggle_button_get_active(button);
+
+	gtk_widget_set_sensitive(add_dialog->label_least, !toggle_button_active);
+	gtk_widget_set_sensitive(add_dialog->label_best, !toggle_button_active);
+	gtk_widget_set_sensitive(add_dialog->compression_scale, !toggle_button_active);
 }
 
-static void fix_adjustment_value (GtkAdjustment *adjustment, gpointer user_data)
+static void fix_adjustment_value (GtkAdjustment *adjustment, XArchive *archive)
 {
-	unsigned short int digit = gtk_adjustment_get_value (adjustment);
-	if (digit & 1)
+	gushort value = gtk_adjustment_get_value(adjustment);
+
+	if ((value - archive->compressor.least) % archive->compressor.steps == 0)
 		return;
 	else
-		gtk_adjustment_set_value (adjustment, digit-1);
+		gtk_adjustment_set_value(adjustment, value - 1);
 }
 
 Add_dialog_data *xa_create_add_dialog()
@@ -69,7 +85,7 @@ Add_dialog_data *xa_create_add_dialog()
 	GTK_COMPAT_TOOLTIPS;
 	GtkWidget *dialog_vbox1,*label1,*label2,*label4,*label5,*label7,*hbox1,*hbox2,*hbox3,*hbox4;
 	GtkWidget *alignment1, *alignment2, *alignment3, *vbox3, *frame2, *frame3, *frame4, *alignment4;
-	GtkWidget *vbox1;
+	GtkWidget *vbox1, *vbox2, *table;
 	Add_dialog_data *add_dialog;
 	GSList *group;
 
@@ -85,6 +101,7 @@ Add_dialog_data *xa_create_add_dialog()
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(add_dialog->dialog1));
 
 	add_dialog->notebook1 = gtk_notebook_new ();
+	gtk_widget_set_can_focus(add_dialog->notebook1, FALSE);
 	gtk_container_add(GTK_CONTAINER(dialog_vbox1), add_dialog->notebook1);
 	gtk_container_set_border_width (GTK_CONTAINER (add_dialog->notebook1),4);
 
@@ -99,28 +116,30 @@ Add_dialog_data *xa_create_add_dialog()
 	gtk_box_pack_start (GTK_BOX (vbox1), add_dialog->filechooserwidget1, TRUE, TRUE,2);
 
 	add_dialog->frame1 = gtk_frame_new (NULL);
-	gtk_box_pack_start (GTK_BOX (vbox1), add_dialog->frame1, FALSE, TRUE,5);
-	gtk_container_set_border_width (GTK_CONTAINER (add_dialog->frame1),5);
+	gtk_box_pack_start(GTK_BOX(vbox1), add_dialog->frame1, FALSE, FALSE, 4);
+	gtk_container_set_border_width(GTK_CONTAINER(add_dialog->frame1), 4);
 
 	alignment1 = gtk_alignment_new (0.5, 0.5, 1, 1);
 	gtk_container_add (GTK_CONTAINER (add_dialog->frame1), alignment1);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1), 0, 0, 20, 20);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 6, 48, 48);
 
-	add_dialog->label = gtk_label_new(_("File Paths: "));
+	add_dialog->label = gtk_label_new(_("File Paths"));
 	gtk_frame_set_label_widget(GTK_FRAME(add_dialog->frame1), add_dialog->label);
 
 	hbox1 = gtk_hbox_new (TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment1), hbox1);
 
-	add_dialog->store_path = gtk_radio_button_new_with_mnemonic (NULL, _("Store full paths"));
+	add_dialog->store_path = gtk_radio_button_new_with_mnemonic(NULL, _("With full path"));
 	gtk_box_pack_start (GTK_BOX (hbox1), add_dialog->store_path, FALSE, FALSE, 0);
 	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(add_dialog->store_path));
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->store_path), FALSE);
+	g_signal_connect(G_OBJECT(add_dialog->store_path), "focus", G_CALLBACK(no_focus), NULL);
 
-	add_dialog->no_store_path = gtk_radio_button_new_with_mnemonic(group, _("Do not store paths"));
+	add_dialog->no_store_path = gtk_radio_button_new_with_mnemonic(group, _("Without parent path"));
 	gtk_box_pack_start (GTK_BOX (hbox1), add_dialog->no_store_path, FALSE, FALSE, 0);
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->no_store_path), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_dialog->no_store_path),TRUE);
+	g_signal_connect(G_OBJECT(add_dialog->no_store_path), "focus", G_CALLBACK(no_focus), NULL);
 
 	/* Options page */
 	add_dialog->option_notebook_vbox = gtk_vbox_new (FALSE, 0);
@@ -129,9 +148,9 @@ Add_dialog_data *xa_create_add_dialog()
 	label2 = gtk_label_new (_("Options"));
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (add_dialog->notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (add_dialog->notebook1), 1), label2);
 
-	hbox2 = gtk_hbox_new (TRUE, 10);
+	hbox2 = gtk_hbox_new(TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (add_dialog->option_notebook_vbox), hbox2, TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 5);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox2), 6);
 
 	frame4 = gtk_frame_new (NULL);
 	gtk_box_pack_start (GTK_BOX (hbox2), frame4, TRUE, TRUE, 0);
@@ -144,17 +163,24 @@ Add_dialog_data *xa_create_add_dialog()
 	vbox3 = gtk_vbox_new (TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment4),vbox3);
 
-	add_dialog->update = gtk_check_button_new_with_mnemonic (_("Update and add"));
+	add_dialog->overwrite = gtk_check_button_new_with_mnemonic(_("Overwrite existing files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(add_dialog->overwrite), FALSE);
+	gtk_box_pack_start(GTK_BOX(vbox3), add_dialog->overwrite, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(add_dialog->overwrite), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
+	g_signal_connect(G_OBJECT(add_dialog->overwrite), "focus", G_CALLBACK(no_focus), NULL);
+
+	add_dialog->update = gtk_check_button_new_with_mnemonic(_("Update existing files and add new ones"));
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->update), FALSE);
 	gtk_widget_set_tooltip_text(add_dialog->update, _("This option will add any new files and update any files which are already in the archive but older there"));
 	gtk_box_pack_start (GTK_BOX (vbox3), add_dialog->update, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(add_dialog->update), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
 
-	add_dialog->freshen = gtk_check_button_new_with_mnemonic (_("Freshen and replace"));
+	add_dialog->freshen = gtk_check_button_new_with_mnemonic(_("Freshen existing files only"));
 
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->freshen), FALSE);
 	gtk_widget_set_tooltip_text(add_dialog->freshen, _("This option will only add files which are already in the archive but older there; unlike the update option it will not add any new files"));
 	gtk_box_pack_start (GTK_BOX (vbox3), add_dialog->freshen, FALSE, FALSE, 0);
-	g_signal_connect (G_OBJECT (add_dialog->freshen),"toggled",G_CALLBACK (add_fresh_update_toggled_cb) , add_dialog);
+	g_signal_connect(G_OBJECT(add_dialog->freshen), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), add_dialog);
 
 	add_dialog->recurse = gtk_check_button_new_with_mnemonic (_("Include subdirectories"));
 	gtk_button_set_focus_on_click (GTK_BUTTON (add_dialog->recurse), FALSE);
@@ -169,22 +195,42 @@ Add_dialog_data *xa_create_add_dialog()
 	gtk_widget_set_tooltip_text(add_dialog->solid_archive, _("In a solid archive the files are grouped together resulting in a better compression ratio"));
 	gtk_box_pack_start (GTK_BOX (vbox3), add_dialog->solid_archive, FALSE, FALSE, 0);
 
-	label7 = gtk_label_new (_("Actions: "));
+	label7 = gtk_label_new(_("Actions"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame4), label7);
 
-	hbox3 = gtk_hbox_new (TRUE, 10);
+	hbox3 = gtk_hbox_new(TRUE, 8);
 	gtk_box_pack_start (GTK_BOX (add_dialog->option_notebook_vbox), hbox3, TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox3), 5);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox3), 6);
 
 	frame2 = gtk_frame_new (NULL);
 	gtk_box_pack_start (GTK_BOX (hbox3), frame2, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame2), GTK_SHADOW_OUT);
 
-	add_dialog->alignment2 = gtk_alignment_new(0.5, 0.5, 1, 0);
-	gtk_container_add (GTK_CONTAINER (frame2), add_dialog->alignment2);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (add_dialog->alignment2), 0, 0, 5, 5);
+	vbox2 = gtk_vbox_new(TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(frame2), vbox2);
 
-	label4 = gtk_label_new (_("Compression: "));
+	add_dialog->alignment2 = gtk_alignment_new(0.5, 1, 1, 0);
+	gtk_container_add(GTK_CONTAINER(vbox2), add_dialog->alignment2);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(add_dialog->alignment2), 0, 0, 12, 12);
+
+	table = gtk_table_new(2, 2, FALSE);
+	gtk_container_add(GTK_CONTAINER(vbox2), table);
+	add_dialog->label_least = gtk_label_new(_("least"));
+	gtk_misc_set_alignment(GTK_MISC(add_dialog->label_least), 0, 0);
+	gtk_misc_set_padding(GTK_MISC(add_dialog->label_least), 10, 0);
+	gtk_table_attach(GTK_TABLE(table), add_dialog->label_least, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
+	add_dialog->label_best = gtk_label_new(_("best"));
+	gtk_misc_set_alignment(GTK_MISC(add_dialog->label_best), 1, 0);
+	gtk_misc_set_padding(GTK_MISC(add_dialog->label_best), 10, 0);
+	gtk_table_attach(GTK_TABLE(table), add_dialog->label_best, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
+
+	add_dialog->uncompressed_button = gtk_check_button_new_with_label(_("No compression"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(add_dialog->uncompressed_button), FALSE);
+	gtk_button_set_alignment(GTK_BUTTON(add_dialog->uncompressed_button), 0.5, 1);
+	gtk_table_attach(GTK_TABLE(table), add_dialog->uncompressed_button, 0, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 24);
+	g_signal_connect(G_OBJECT(add_dialog->uncompressed_button), "toggled", G_CALLBACK(toggle_compression), add_dialog);
+
+	label4 = gtk_label_new(_("Compression"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame2), label4);
 
 	frame3 = gtk_frame_new (NULL);
@@ -193,23 +239,23 @@ Add_dialog_data *xa_create_add_dialog()
 
 	alignment3 = gtk_alignment_new(0.5, 0.5, 1, 0);
 	gtk_container_add (GTK_CONTAINER (frame3), alignment3);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment3), 0, 0, 5, 5);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment3), 0, 0, 12, 12);
 
-	hbox4 = gtk_hbox_new (FALSE, 0);
+	hbox4 = gtk_hbox_new(FALSE, 4);
 	gtk_container_add (GTK_CONTAINER (alignment3), hbox4);
 
 	add_dialog->add_password = gtk_check_button_new_with_mnemonic (_("Password:"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(add_dialog->add_password), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox4), add_dialog->add_password, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (add_dialog->add_password), "toggled",G_CALLBACK (password_toggled_cb) , add_dialog);
 
 	add_dialog->add_password_entry = gtk_entry_new ();
-	gtk_box_pack_start (GTK_BOX (hbox4), add_dialog->add_password_entry, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox4), add_dialog->add_password_entry, TRUE, TRUE, 0);
 	gtk_entry_set_visibility (GTK_ENTRY (add_dialog->add_password_entry), FALSE);
-	gtk_entry_set_width_chars (GTK_ENTRY (add_dialog->add_password_entry), 15);
 	gtk_widget_set_sensitive (add_dialog->add_password_entry, FALSE);
 	gtk_entry_set_activates_default(GTK_ENTRY(add_dialog->add_password_entry), TRUE);
 
-	label5 = gtk_label_new (_("Encryption: "));
+	label5 = gtk_label_new(_("Encryption"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame3), label5);
 
 	add_dialog->cancel_button = gtk_button_new_from_stock ("gtk-cancel");
@@ -223,8 +269,8 @@ Add_dialog_data *xa_create_add_dialog()
 	alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
 	gtk_container_add (GTK_CONTAINER (alignment2), add_dialog->add_hbox);
 
-	gtk_box_pack_start(GTK_BOX(add_dialog->add_hbox), add_dialog->add_image, FALSE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(add_dialog->add_hbox), add_dialog->add_label, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(add_dialog->add_hbox), add_dialog->add_image, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(add_dialog->add_hbox), add_dialog->add_label, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(add_dialog->add_button), alignment2);
 
 	gtk_dialog_add_action_widget (GTK_DIALOG (add_dialog->dialog1), add_dialog->add_button, GTK_RESPONSE_OK);
@@ -236,18 +282,14 @@ Add_dialog_data *xa_create_add_dialog()
 void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 {
 	GTK_COMPAT_TOOLTIPS;
-	gboolean epub, flag = FALSE;
-	gchar *compression_msg = NULL;
-	gushort default_value, max_value;
-	default_value = max_value = 0;
+	gboolean epub, full_path, normal;
+	gchar *compression_msg;
 
 	if (progress)
 		gtk_widget_hide(progress->window);
 
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_save_geometry)) && prefs_window->add_coords[0] != -1)
 		gtk_window_set_default_size (GTK_WINDOW(add_dialog->dialog1), prefs_window->add_coords[0], prefs_window->add_coords[1]);
-	else
-		gtk_widget_set_size_request (add_dialog->dialog1,530,420);
 
 	prefs_window->size_changed[1] = TRUE;
 
@@ -258,85 +300,55 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 	if ((archive->location_path && *archive->location_path) || !archive->can_full_path[1] || epub)
 	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_dialog->no_store_path), TRUE);
-		flag = FALSE;
+		full_path = FALSE;
 	}
 	else
-		flag = TRUE;
-
-	gtk_widget_set_sensitive(add_dialog->label, flag);
-	gtk_widget_set_sensitive(add_dialog->store_path,flag);
-
-	g_signal_connect (G_OBJECT (add_dialog->update),"toggled",G_CALLBACK (add_update_fresh_toggled_cb) , add_dialog);
+		full_path = TRUE;
 
+	gtk_widget_set_sensitive(add_dialog->label, full_path);
+	gtk_widget_set_sensitive(add_dialog->store_path, full_path);
 	gtk_widget_set_sensitive(add_dialog->update, archive->can_update[1]);
 	gtk_widget_set_sensitive(add_dialog->freshen, archive->can_freshen[1]);
 	gtk_widget_set_sensitive(add_dialog->recurse, TRUE);
 	gtk_widget_set_sensitive(add_dialog->remove_files, archive->can_move);
 	gtk_widget_set_sensitive(add_dialog->solid_archive, archive->can_solid);
 
-		if (archive->type == XARCHIVETYPE_7ZIP)
-		{
-			compression_msg = _("0 = no compression, 5 is default, 9 = best compression but slowest");
-			default_value = 5;
-			max_value = 9;
-		}
-		else if (archive->type == XARCHIVETYPE_ZIP)
-		{
-			compression_msg = _("0 = no compression, 6 is default, 9 = best compression but slowest");
-			default_value = 6;
-			max_value = 9;
-		}
-		else if (archive->type == XARCHIVETYPE_RAR)
-		{
-			compression_msg = _("0 = no compression, 3 is default, 5 = best compression but slowest");
-			default_value = 3;
-			max_value = 5;
-		}
-		else if (archive->type == XARCHIVETYPE_ARJ)
-		{
-			compression_msg = _("0 = no compression, 1 is default, 4 = fastest but least compression");
-			default_value = 1;
-			max_value = 4;
-		}
-		else if (archive->type == XARCHIVETYPE_LHA)
-		{
-			compression_msg = _("5 = default compression, 7 = max compression");
-			default_value = 5;
-			max_value = 7;
-		}
-		if (archive->type == XARCHIVETYPE_XZ)
-		{
-			compression_msg = _("0 = no compression, 5 is default, 9 = best compression but slowest");
-			default_value = 5;
-			max_value = 9;
-		}
+	if ((!gtk_widget_is_sensitive(add_dialog->update) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->update))) ||
+	    (!gtk_widget_is_sensitive(add_dialog->freshen) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->freshen))))
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_dialog->overwrite), TRUE);
 
-	flag = (compression_msg != NULL);
+	normal = (archive->compressor.least <= archive->compressor.best);
 
-	if (archive->type == XARCHIVETYPE_7ZIP)
-		compression_value = gtk_adjustment_new(default_value, 0, max_value, 2, 2, 0);
-	else if (archive->type == XARCHIVETYPE_LHA)
-		compression_value = gtk_adjustment_new(default_value, 5, max_value, 7, 7, 0);
+	if (normal)
+		compression_value = gtk_adjustment_new(archive->compression, archive->compressor.least, archive->compressor.best, archive->compressor.steps, archive->compressor.steps, 0);
 	else
-		compression_value = gtk_adjustment_new(default_value, 0, max_value, 0, 0, 0);
+		compression_value = gtk_adjustment_new(archive->compression, archive->compressor.best, archive->compressor.least, archive->compressor.steps, archive->compressor.steps, 0);
 
 	add_dialog->compression_scale = gtk_hscale_new(GTK_ADJUSTMENT(compression_value));
-	if (gtk_bin_get_child(GTK_BIN(add_dialog->alignment2)) == NULL)
+
+	if (!gtk_bin_get_child(GTK_BIN(add_dialog->alignment2)))
 		gtk_container_add (GTK_CONTAINER (add_dialog->alignment2), add_dialog->compression_scale);
+
 	gtk_scale_set_value_pos (GTK_SCALE (add_dialog->compression_scale), GTK_POS_TOP);
 	gtk_scale_set_digits (GTK_SCALE (add_dialog->compression_scale), 0);
+	gtk_range_set_inverted(GTK_RANGE(add_dialog->compression_scale), !normal);
+	gtk_widget_set_sensitive(add_dialog->compression_scale, archive->can_compress);
 
-	if (archive->compression == 0)
-		archive->compression = default_value;
+	gtk_widget_set_sensitive(add_dialog->uncompressed_button, archive->can_compress && archive->compressor.can_uncompressed);
 
-	gtk_widget_set_sensitive(add_dialog->compression_scale,flag);
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(compression_value), archive->compression);
+	if (gtk_widget_get_sensitive(add_dialog->uncompressed_button) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->uncompressed_button)))
+	{
+		gtk_widget_set_sensitive(add_dialog->label_least, FALSE);
+		gtk_widget_set_sensitive(add_dialog->label_best, FALSE);
+		gtk_widget_set_sensitive(add_dialog->compression_scale, FALSE);
+	}
 
-	if (archive->type == XARCHIVETYPE_ARJ)
-		gtk_range_set_inverted (GTK_RANGE (add_dialog->compression_scale), TRUE);
-	else if (archive->type == XARCHIVETYPE_7ZIP)
-		g_signal_connect(G_OBJECT(compression_value), "value-changed", G_CALLBACK(fix_adjustment_value), NULL);
+	if (archive->compressor.steps > 1)
+		g_signal_connect(G_OBJECT(compression_value), "value-changed", G_CALLBACK(fix_adjustment_value), archive);
+
+	compression_msg = g_strdup_printf(_("%hu is least compression\n%hu is default compression\n%hu is best compression"), archive->compressor.least, archive->compressor.preset, archive->compressor.best);
 	gtk_widget_set_tooltip_text(add_dialog->compression_scale, compression_msg);
+	g_free(compression_msg);
 
 	gtk_widget_set_sensitive(add_dialog->add_password, archive->can_password && !epub);
 	gtk_widget_set_sensitive(add_dialog->add_password_entry, archive->can_password && !epub);
@@ -346,7 +358,6 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 {
 	gchar *temp_password = NULL;
-	gchar *compression = NULL;
 	gboolean done = FALSE;
 	GSList *list = NULL;
 
@@ -411,27 +422,28 @@ void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 				archive->do_solid = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->solid_archive));
 
 			if (gtk_widget_is_sensitive(add_dialog->compression_scale))
-			{
 				archive->compression = gtk_adjustment_get_value(GTK_ADJUSTMENT(compression_value));
-				compression = g_strdup_printf("%hu", archive->compression);
-			}
+			else
+				archive->compression = 0;
+
 			gtk_widget_hide(add_dialog->dialog1);
 
 			if (!archive->do_full_path)
 				archive->child_dir = g_path_get_dirname(list->data);
 
-			xa_execute_add_commands(archive, list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse)), compression);
-			g_free(compression);
+			xa_execute_add_commands(archive, list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse)));
 		}
 	}
+	gtk_widget_destroy(add_dialog->compression_scale);
 	gtk_widget_hide (add_dialog->dialog1);
 }
 
-void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse, gchar *compression)
+void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse)
 {
 	gchar *new_path = NULL;
-	gchar *esc2, *basedir;
-	gboolean result = FALSE;
+	gchar *esc2, *dest, *org_path = NULL;
+	gboolean success;
+	gint result, n_list = 0;
 	GString *items;
 	gchar *command = NULL;
 	GSList *slist = NULL;
@@ -440,38 +452,73 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse,
 
 	if (xa_main_window)
 	{
-		gtk_label_set_text(GTK_LABEL(total_label),_("Adding files to archive, please wait..."));
 		/* This in case the user wants to add files in a directory in the archive tree */
 		if (archive->location_path != NULL)
 		{
-			result = xa_create_working_directory(archive);
-			if (result == FALSE)
+			success = xa_create_working_directory(archive);
+
+			if (!success)
 				return;
 
 			items = g_string_new("");
 			new_path = g_strconcat(archive->working_dir, "/", archive->location_path, NULL);
 			result = g_mkdir_with_parents(new_path,0700);
-			if (result < 0)
+
+			if (result != 0)
 			{
 				g_free(new_path);
 				return;
 			}
+
 			slist = list;
-			basedir = g_path_get_dirname(slist->data);
+
 			while (slist)
 			{
 				esc2 = g_shell_quote(slist->data);
 				g_string_append(items,esc2);
 				g_string_append_c(items,' ');
 				g_free(esc2);
+				n_list++;
 				slist = slist->next;
 			}
-			g_free(basedir);
-			command = g_strconcat ("cp -rf ",items->str," ",new_path,NULL);
-			g_free(new_path);
+
+			dest = xa_escape_bad_chars(new_path, ESCAPES);
+			command = g_strconcat("cp -rfp -- ", items->str, dest, NULL);
+			g_free(dest);
 			g_string_free(items,TRUE);
-			xa_run_command(archive, command);
+
+			if ((n_list == 1) && g_file_test(list->data, G_FILE_TEST_IS_REGULAR))
+				org_path = g_path_get_dirname(list->data);
+
+			xa_remove_slash_from_path(new_path);
+
+			if (g_strcmp0(org_path, new_path) == 0)
+				success = TRUE;
+			else
+			{
+				archive->status = XARCHIVESTATUS_ADD;
+				success = xa_run_command(archive, command);
+			}
+
+			g_free(org_path);
+			g_free(new_path);
 			g_free(command);
+
+			if (!success)
+			{
+				new_path = xa_remove_level_from_path(archive->location_path);
+				g_free(archive->location_path);
+
+				if (strcmp(new_path, ".") == 0)
+					archive->location_path = NULL;
+				else
+					archive->location_path = g_strconcat(new_path, "/", NULL);
+
+				xa_set_location_entry(archive);
+
+				g_free(new_path);
+				return;
+			}
 		}
 		xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
 	}
@@ -484,12 +531,18 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse,
 	files = xa_collect_filenames(archive, dirlist);
 	g_slist_free_full(dirlist, g_free);
 
+	if (archive->location_path != NULL)
+	{
+		g_free(archive->child_dir);
+		archive->child_dir = g_strdup(archive->working_dir);
+	}
+
 	archive->status = XARCHIVESTATUS_ADD;
-	(*archive->archiver->add)(archive, files, compression);
+	(*archive->archiver->add)(archive, files);
 
 	g_free(archive->child_dir);
 	archive->child_dir = NULL;
 
-	if (xa_main_window)
+	if (xa_main_window && (archive->status == XARCHIVESTATUS_IDLE))
 		xa_reload_archive_content(archive);
 }
