@@ -63,8 +63,7 @@ static gboolean xa_process_stdout (GIOChannel *ioc, GIOCondition cond, XArchive
 
 			g_free(line);
 
-			while (gtk_events_pending())
-				gtk_main_iteration();
+			process_gtk_events();
 
 			return TRUE;
 		}
@@ -414,6 +413,7 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 	if (xa_main_window)
 	{
 		gtk_widget_set_sensitive(Stop_button, TRUE);
+		xa_show_archive_status(archive);
 
 		if (!archive->timeout)
 			archive->timeout = g_timeout_add(350, (GSourceFunc) xa_flash_led_indicator, archive);
@@ -426,11 +426,8 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 			archive->timeout = g_timeout_add(100, (GSourceFunc) xa_pulse_progress_bar, archive);
 	}
 
-	if (archive->output)
-	{
-		g_slist_free_full(archive->output, g_free);
-		archive->output = NULL;
-	}
+	g_slist_free_full(archive->output, g_free);
+	archive->output = NULL;
 
 	ioc = g_io_channel_unix_new(archive->child_fdout);
 	g_io_channel_set_encoding(ioc, NULL, NULL);
@@ -484,11 +481,10 @@ void xa_clean_archive_structure (XArchive *archive)
 	if (archive->comment)
 		g_string_free(archive->comment, TRUE);
 
-	if (archive->output)
-		g_slist_free_full(archive->output, g_free);
+	g_slist_free_full(archive->output, g_free);
 
-	if (archive->clipboard)
-		xa_clipboard_clear(NULL, archive);
+	if (archive == XA_Clipboard.archive)
+		 xa_clipboard_clear();
 
 	g_free(archive->column_types);
 	g_free(archive->path[0]);
@@ -595,9 +591,7 @@ gboolean xa_run_command (XArchive *archive, const gchar *command)
 		while (pid != archive->child_pid && pid != -1)
 		{
 			pid = waitpid(archive->child_pid, &status, WNOHANG);
-
-			while (gtk_events_pending())
-				gtk_main_iteration();
+			process_gtk_events();
 		}
 
 		if (WIFEXITED(status))
@@ -644,7 +638,7 @@ gint xa_find_archive_index (gint page_num)
 	return -1;
 }
 
-gint xa_get_new_archive_idx()
+gint xa_get_new_archive_index ()
 {
 	gint i;
 
@@ -653,6 +647,7 @@ gint xa_get_new_archive_idx()
 		if (archive[i] == NULL)
 			return i;
 	}
+
 	return -1;
 }
 
@@ -810,10 +805,7 @@ void xa_detect_encrypted_archive (XArchive *archive)
 	(*archive->archiver->list)(archive);
 
 	do
-	{
-		while (gtk_events_pending())
-			gtk_main_iteration();
-	}
+		process_gtk_events();
 	while (archive->child_ref);
 }
 
@@ -841,7 +833,7 @@ void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer user_dat
 
 	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
-	if ((idx >= 0) && gtk_tree_selection_get_selected(selection, &model, &iter))
+	if ((idx != -1) && gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
 		path = gtk_tree_model_get_path(model,&iter);
 		if ( ! gtk_tree_view_row_expanded(GTK_TREE_VIEW(archive_dir_treeview),path))
