@@ -69,6 +69,7 @@ typedef enum
 	XARCHIVESTATUS_DELETE,
 	XARCHIVESTATUS_SFX,
 	XARCHIVESTATUS_RELOAD,
+	XARCHIVESTATUS_OTHER,
 	XARCHIVESTATUS_ERROR
 } XArchiveStatus;
 
@@ -87,7 +88,7 @@ typedef void (*list_func)(XArchive *);
 typedef void (*parse_output_func)(gchar *, XArchive *);
 typedef void (*test_func)(XArchive *);
 typedef gboolean (*extract_func)(XArchive *, GSList *);
-typedef void (*add_func)(XArchive *, GSList *, gchar *);
+typedef void (*add_func)(XArchive *, GSList *);
 typedef void (*delete_func)(XArchive *, GSList *);
 
 typedef struct
@@ -124,7 +125,14 @@ struct _XEntry
 	XEntry *next;
 };
 
-typedef struct _XAClipboard XAClipboard;
+typedef struct
+{
+	gboolean can_uncompressed;   // has compression level 0?
+	gushort least;               // least compression level
+	gushort preset;              // default compression level
+	gushort best;                // best compression level
+	gushort steps;               // level steps (usually 1)
+} compressor_t;
 
 struct _XArchive
 {
@@ -148,7 +156,6 @@ struct _XArchive
 	GtkCellRenderer *text_renderer;
 	GSList *back;
 	GSList *forward;
-	XAClipboard *clipboard;
 	/* environment */
 	gchar *path[4];          // 0: original, 1: escaped, 2: working copy, 3: escaped working copy
 	gchar *working_dir;
@@ -162,6 +169,7 @@ struct _XArchive
 	gchar *password;
 	gboolean has_comment;
 	GString *comment;
+	compressor_t compressor;
 	gushort compression;
 	/* capabilities */           // 0: extract, 1: add
 	gboolean can_test;           // can test an archive for integrity
@@ -178,22 +186,23 @@ struct _XArchive
 	gboolean can_freshen[2];     // can limit activity to only changed files
 	gboolean can_move;           // can delete files after adding
 	gboolean can_solid;          // can create a solid archive
+	gboolean can_compress;       // can compress at different levels
 	/* instructions */
-	gboolean do_full_path;
-	gboolean do_touch;
-	gboolean do_overwrite;
-	gboolean do_update;
-	gboolean do_freshen;
-	gboolean do_move;
-	gboolean do_solid;
+	gboolean do_full_path;   // extract_func, (xa_execute_add_commands)
+	gboolean do_touch;       // extract_func
+	gboolean do_overwrite;   // extract_func
+	gboolean do_update;      // extract_func, add_func
+	gboolean do_freshen;     // extract_func, add_func
+	gboolean do_move;        // add_func
+	gboolean do_solid;       // add_func
 	/* child process */
 	guint timeout;
-	gchar *child_dir;     // (free and set NULL after use)
+	gchar *child_dir;   // (free and set NULL after use)
 	GPid child_pid;
 	guint child_ref;
 	gint child_fdout;
 	gint child_fderr;
-	gchar *command;       // (free before use)
+	gchar *command;     // (free before use)
 	GSList *output;
 	/* (un)compressor interface */
 	XArchiver *archiver;
@@ -205,21 +214,26 @@ extern XArchive *archive[];
 
 #define can_rename(archive) ((archive)->can_extract && (archive)->can_delete && (archive)->can_add)
 
-#define XA_CLIPBOARD (gdk_atom_intern_static_string("XARCHIVER_OWN_CLIPBOARD"))
-#define XA_INFO_LIST (gdk_atom_intern_static_string("application/xarchiver-info-list"))
-
 typedef enum
 {
+	XA_CLIPBOARD_EMPTY,
+	XA_CLIPBOARD_COPY,
 	XA_CLIPBOARD_CUT,
-	XA_CLIPBOARD_COPY
+	XA_CLIPBOARD_EDIT
 } XAClipboardMode;
 
-struct _XAClipboard
+typedef struct
 {
 	XAClipboardMode mode;
+	gchar *origin;
+	XEntry *target;
+	GSList *paths;
 	GSList *files;
-	XArchive *target;
-};
+	struct timespec mtime;
+	XArchive *archive;
+} XAClipboard;
+
+extern XAClipboard XA_Clipboard;
 
 gchar *xa_build_full_path_name_from_entry(XEntry *);
 void xa_clean_archive_structure(XArchive *);
@@ -232,7 +246,7 @@ gint xa_find_archive_index(gint);
 XEntry *xa_find_entry_from_dirpath(XArchive *, const gchar *);
 void xa_free_entry(XArchive *, XEntry *);
 gboolean xa_get_compressed_tar_type(XArchiveType *);
-gint xa_get_new_archive_idx();
+gint xa_get_new_archive_index();
 gboolean xa_has_containing_directory(XArchive *);
 XArchive *xa_init_archive_structure(ArchiveType);
 gboolean xa_run_command(XArchive *, const gchar *);
