@@ -35,7 +35,7 @@ static GtkWidget *label_password;
 
 static const GtkTargetEntry drop_targets[] =
 {
-	{ "text/uri-list",0,0 },
+	{"text/uri-list", 0, 0}
 };
 
 static void xa_select_where_to_extract (GtkEntry *entry, gint icon_pos, GTK_COMPAT_ENTRY_ICON_TYPE button_release, gpointer user_data)
@@ -77,6 +77,20 @@ static void xa_toggle_all_files_radio (GtkToggleButton *button, Extract_dialog_d
 	gtk_widget_set_sensitive(dialog->ensure_directory, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->all_files_radio)));
 }
 
+static void xa_toggle_relative_paths_radio (GtkToggleButton *button, Extract_dialog_data *dialog)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->relative_paths_radio)))
+	{
+		gtk_widget_set_sensitive(dialog->touch, FALSE);
+		gtk_widget_set_sensitive(dialog->fresh, FALSE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive(dialog->touch, dialog->archive->can_touch);
+		gtk_widget_set_sensitive(dialog->fresh, dialog->archive->can_freshen[0]);
+	}
+}
+
 static void xa_activate_entry (GtkToggleButton *button, Extract_dialog_data *dialog)
 {
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(dialog->files_radio)))
@@ -111,27 +125,32 @@ static void xa_multi_extract_dialog_selection_changed (GtkTreeSelection *selecti
 
 static void xa_multi_extract_dialog_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *data, guint info, guint time, gpointer dialog_data)
 {
-	gchar **array = NULL;
+	gchar **uris;
 	gchar *filename;
-	unsigned int len = 0;
+	unsigned int n = 0;
 
-	gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
-	array = gtk_selection_data_get_uris (data);
-	if (array == NULL)
+	uris = gtk_selection_data_get_uris(data);
+
+	if (!uris)
 	{
-		xa_show_message_dialog(GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,"",_("Sorry, I could not perform the operation!"));
-		gtk_drag_finish (context,FALSE,FALSE,time);
+		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "", _("Sorry, I could not perform the operation!"));
+		gtk_drag_finish(context, FALSE, FALSE, time);
 		return;
 	}
-	gtk_drag_finish (context,TRUE,FALSE,time);
-	while (array[len])
+
+	while (uris[n])
 	{
-		filename = g_filename_from_uri (array[len],NULL,NULL);
-		xa_multi_extract_dialog_add_file(filename, dialog_data);
-		g_free (filename);
-		len++;
+		filename = g_filename_from_uri(uris[n], NULL, NULL);
+
+		if (filename)
+			xa_multi_extract_dialog_add_file(filename, dialog_data);
+
+		g_free(filename);
+		n++;
 	}
-	g_strfreev (array);
+
+	g_strfreev(uris);
+	gtk_drag_finish(context, TRUE, FALSE, time);
 }
 
 static void xa_multi_extract_dialog_select_files_to_add (GtkButton *button, Multi_extract_data *dialog)
@@ -239,6 +258,9 @@ static gchar *xa_multi_extract_one_archive (Multi_extract_data *dialog, gchar *f
 	archive = xa_init_archive_structure(xa);
 	dialog->archive = archive;
 	archive->do_overwrite = overwrite;
+	archive->do_update = FALSE;
+	archive->do_freshen = FALSE;
+	archive->do_touch = FALSE;
 	archive->do_full_path = full_path;
 	archive->path[0] = g_strdup(filename);
 	archive->path[1] = xa_escape_bad_chars(filename, ESCAPES);
@@ -290,7 +312,7 @@ static void toggle_overwrite_update_freshen (GtkToggleButton *button, Extract_di
 Extract_dialog_data *xa_create_extract_dialog()
 {
 	GTK_COMPAT_TOOLTIPS;
-	GSList *radiobutton1_group;
+	GSList *radiobutton1_group, *radiobutton2_group;
 	Extract_dialog_data *dialog_data;
 	GtkWidget *hbox1, *hbox2, *hbox3, *vbox1, *vbox2, *vbox3, *vbox5, *alignment, *alignment1, *alignment2, *alignment3, *label1, *label2, *label3;
 	GtkWidget *frame1, *frame2;
@@ -308,12 +330,14 @@ Extract_dialog_data *xa_create_extract_dialog()
 	dialog_data->dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog_data->dialog1));
 
 	vbox1 = gtk_vbox_new (FALSE,2);
-	gtk_container_add(GTK_CONTAINER(dialog_data->dialog_vbox1), vbox1);
+	gtk_box_pack_start(GTK_BOX(dialog_data->dialog_vbox1), vbox1, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1),2);
 
 	label1 = gtk_label_new (_("Extract to:"));
-	gtk_box_pack_start (GTK_BOX (vbox1),label1,FALSE,FALSE,0);
-	gtk_misc_set_alignment (GTK_MISC (label1),0,0.5);
+	alignment = gtk_alignment_new(0, 0.5, 0, 0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 2, 2, 0, 0);
+	gtk_container_add(GTK_CONTAINER(alignment), label1);
+	gtk_box_pack_start(GTK_BOX(vbox1), alignment, FALSE, FALSE, 0);
 
 	dialog_data->destination_path_entry = GTK_COMPAT_ENTRY_ICON_NEW();
 	GTK_COMPAT_ENTRY_ICON(dialog_data->destination_path_entry, xa_select_where_to_extract, dialog_data);
@@ -321,9 +345,10 @@ Extract_dialog_data *xa_create_extract_dialog()
 	gtk_entry_set_activates_default(GTK_ENTRY(dialog_data->destination_path_entry), TRUE);
 
 	alignment = gtk_alignment_new(0.5, 0.5, 1, 1);
-	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 14, 0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 14, 2);
 
 	dialog_data->ensure_directory = gtk_check_button_new_with_mnemonic(_("Ensure a containing directory"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->ensure_directory), FALSE);
 	gtk_widget_set_tooltip_text(dialog_data->ensure_directory, _("Ensure that the contents of the extracted archive is always in a containing directory"));
 	gtk_container_add(GTK_CONTAINER(alignment), dialog_data->ensure_directory);
 	gtk_box_pack_start(GTK_BOX(vbox1), alignment, FALSE, FALSE, 0);
@@ -340,75 +365,108 @@ Extract_dialog_data *xa_create_extract_dialog()
 
 	alignment1 = gtk_alignment_new (0.5,0.5,1,1);
 	gtk_container_add (GTK_CONTAINER (frame1),alignment1);
-	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 2, 12, 0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 2, 12, 2);
 
-	vbox3 = gtk_vbox_new (FALSE,0);
+	vbox3 = gtk_vbox_new(TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment1),vbox3);
 
 	dialog_data->all_files_radio = gtk_radio_button_new_with_mnemonic (NULL,_("All files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->all_files_radio), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->all_files_radio,FALSE,FALSE,0);
 	radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dialog_data->all_files_radio));
 	g_signal_connect(dialog_data->all_files_radio, "toggled", G_CALLBACK(xa_toggle_all_files_radio), dialog_data);
 
 	dialog_data->selected_radio = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Selected files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->selected_radio), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->selected_radio,FALSE,FALSE,0);
 	radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dialog_data->selected_radio));
 
-	hbox2 = gtk_hbox_new (FALSE,0);
+	hbox2 = gtk_hbox_new(FALSE, 4);
 	gtk_box_pack_start (GTK_BOX (vbox3),hbox2,FALSE,FALSE,0);
 
-	dialog_data->files_radio = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Files: "));
+	dialog_data->files_radio = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Files:"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->files_radio), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox2),dialog_data->files_radio,FALSE,FALSE,0);
 
 	dialog_data->entry2 = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (hbox2),dialog_data->entry2,TRUE,TRUE,0);
-	gtk_entry_set_width_chars (GTK_ENTRY (dialog_data->entry2),10);
 	gtk_widget_set_sensitive(dialog_data->entry2,FALSE);
 	g_signal_connect (G_OBJECT (dialog_data->files_radio),"toggled",G_CALLBACK(xa_activate_entry),dialog_data);
 
-	label2 = gtk_label_new (_("Files "));
+	label2 = gtk_label_new(_("Files"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame1),label2);
 
+	frame1 = gtk_frame_new(_("File Paths"));
+	gtk_box_pack_start(GTK_BOX(vbox2), frame1, TRUE, TRUE, 0);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame1), GTK_SHADOW_OUT);
+
+	alignment1 = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_container_add(GTK_CONTAINER(frame1), alignment1);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 0, 12, 0);
+
+	vbox3 = gtk_vbox_new(TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(alignment1), vbox3);
+
+	dialog_data->full_paths_radio = gtk_radio_button_new_with_mnemonic(NULL, _("With full path"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->full_paths_radio), FALSE);
+	gtk_widget_set_tooltip_text(dialog_data->full_paths_radio, _("The archive's complete directory structure is recreated in the extraction directory"));
+	radiobutton2_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(dialog_data->full_paths_radio));
+	gtk_box_pack_start(GTK_BOX(vbox3), dialog_data->full_paths_radio, FALSE, FALSE, 0);
+
+	dialog_data->relative_paths_radio = gtk_radio_button_new_with_mnemonic(radiobutton2_group, _("Without parent path"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->relative_paths_radio), FALSE);
+	gtk_widget_set_tooltip_text(dialog_data->relative_paths_radio, _("The archive's directory structure is recreated in the extraction directory, but with the parent directories of the selected files removed"));
+	radiobutton2_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(dialog_data->relative_paths_radio));
+	gtk_box_pack_start(GTK_BOX(vbox3), dialog_data->relative_paths_radio, FALSE, FALSE, 0);
+	g_signal_connect(dialog_data->relative_paths_radio, "toggled", G_CALLBACK(xa_toggle_relative_paths_radio), dialog_data);
+
+	dialog_data->no_paths_radio = gtk_radio_button_new_with_mnemonic(radiobutton2_group, _("Without any path"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->no_paths_radio), FALSE);
+	gtk_widget_set_tooltip_text(dialog_data->no_paths_radio, _("The archive's directory structure is not recreated; all files are placed in the extraction directory"));
+	radiobutton2_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(dialog_data->no_paths_radio));
+	gtk_box_pack_start(GTK_BOX(vbox3), dialog_data->no_paths_radio, FALSE, FALSE, 0);
+
 	frame2 = gtk_frame_new (NULL);
 	gtk_box_pack_start (GTK_BOX (vbox2),frame2,TRUE,TRUE,0);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame2),GTK_SHADOW_OUT);
 
 	alignment2 = gtk_alignment_new (0.5,0.5,1,1);
 	gtk_container_add (GTK_CONTAINER (frame2),alignment2);
-	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 0, 4, 12, 0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 0, 2, 12, 2);
 
-	vbox5 = gtk_vbox_new (FALSE,0);
+	vbox5 = gtk_vbox_new(TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment2),vbox5);
 
-	dialog_data->extract_full = gtk_check_button_new_with_mnemonic (_("Extract files with full path"));
-	gtk_widget_set_tooltip_text(dialog_data->extract_full, _("The archive's directory structure is recreated in the extraction directory"));
-	gtk_box_pack_start (GTK_BOX (vbox5),dialog_data->extract_full,FALSE,FALSE,0);
-
 	dialog_data->touch = gtk_check_button_new_with_mnemonic (_("Touch files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->touch), FALSE);
 	gtk_widget_set_tooltip_text(dialog_data->touch, _("When this option is used, the modification times of the files will be the times of extraction instead of the times recorded in the archive"));
 	gtk_box_pack_start (GTK_BOX (vbox5),dialog_data->touch,FALSE,FALSE,0);
 
 	dialog_data->overwrite_check = gtk_check_button_new_with_mnemonic (_("Overwrite existing files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->overwrite_check), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox5),dialog_data->overwrite_check,FALSE,FALSE,0);
 	g_signal_connect(G_OBJECT(dialog_data->overwrite_check), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), dialog_data);
 
 	dialog_data->update = gtk_check_button_new_with_mnemonic (_("Update existing files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->update), FALSE);
 	gtk_widget_set_tooltip_text(dialog_data->update, _("This option performs the same function as the freshen one, extracting files that are newer than those with the same name on disk, and in addition it extracts those files that do not already exist on disk"));
 	gtk_box_pack_start (GTK_BOX (vbox5),dialog_data->update,FALSE,FALSE,0);
 	g_signal_connect(G_OBJECT(dialog_data->update), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), dialog_data);
 
-	dialog_data->fresh = gtk_check_button_new_with_mnemonic (_("Freshen existing files"));
+	dialog_data->fresh = gtk_check_button_new_with_mnemonic(_("Freshen existing files only"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->fresh), FALSE);
 	gtk_widget_set_tooltip_text(dialog_data->fresh, _("Extract only those files that already exist on disk and that are newer than the disk copies"));
 	gtk_box_pack_start (GTK_BOX (vbox5),dialog_data->fresh,FALSE,FALSE,0);
 	g_signal_connect(G_OBJECT(dialog_data->fresh), "toggled", G_CALLBACK(toggle_overwrite_update_freshen), dialog_data);
 
-	label3 = gtk_label_new (_("Options "));
+	label3 = gtk_label_new(_("Options"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame2),label3);
 
 	hbox3 = gtk_hbox_new(FALSE, 4);
-	gtk_box_pack_start (GTK_BOX (vbox5),hbox3,TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(vbox5), hbox3, FALSE, FALSE, 0);
 
 	label_password = gtk_label_new (_("Password:"));
+	gtk_misc_set_padding(GTK_MISC(label_password), 3, 0);
 	gtk_box_pack_start (GTK_BOX (hbox3),label_password,FALSE,FALSE,0);
 
 	dialog_data->password_entry = gtk_entry_new ();
@@ -443,8 +501,6 @@ void xa_set_extract_dialog_options(Extract_dialog_data *dialog_data,gint selecte
 
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_save_geometry))&& prefs_window->extract_dialog[0] != -1)
 		gtk_window_set_default_size (GTK_WINDOW(dialog_data->dialog1),prefs_window->extract_dialog[0],prefs_window->extract_dialog[1]);
-	else
-		gtk_widget_set_size_request (dialog_data->dialog1,-1,370);
 
 	prefs_window->size_changed[0] = TRUE;
 
@@ -463,12 +519,23 @@ void xa_set_extract_dialog_options(Extract_dialog_data *dialog_data,gint selecte
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (dialog_data->all_files_radio),TRUE);
 		}
 
-	gtk_widget_set_sensitive(dialog_data->extract_full, archive->can_full_path[0]);
+	gtk_widget_set_sensitive(dialog_data->full_paths_radio, archive->can_full_path[0]);
+	gtk_widget_set_sensitive(dialog_data->relative_paths_radio, archive->can_full_path[0]);
 	gtk_widget_set_sensitive(dialog_data->touch, archive->can_touch);
 	gtk_widget_set_sensitive(dialog_data->overwrite_check, archive->can_overwrite);
 	gtk_widget_set_sensitive(dialog_data->update, archive->can_update[0]);
 	gtk_widget_set_sensitive(dialog_data->fresh, archive->can_freshen[0]);
 
+	if (!archive->can_full_path[0])
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_data->no_paths_radio), TRUE);
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->relative_paths_radio)))
+		xa_toggle_relative_paths_radio(NULL, dialog_data);
+
+	if ((!gtk_widget_is_sensitive(dialog_data->update) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->update))) ||
+	    (!gtk_widget_is_sensitive(dialog_data->fresh) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->fresh))))
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog_data->overwrite_check), TRUE);
+
 	if (!archive->destination_path)
 	{
 		gchar *archive_dir = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_extract_dir));
@@ -492,13 +559,15 @@ void xa_set_extract_dialog_options(Extract_dialog_data *dialog_data,gint selecte
 	gtk_widget_set_sensitive(label_password, archive->has_password);
 	gtk_widget_set_sensitive(dialog_data->password_entry, archive->has_password);
 
+	dialog_data->archive = archive;
+
 	gtk_widget_show_all(dialog_data->dialog1);
 }
 
 void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dialog_data,GtkTreeSelection *selection)
 {
 	gchar *destination_path, *string;
-	gboolean done = FALSE;
+	gboolean strip, done = FALSE;
 	GSList *names = NULL;
 	GtkTreeModel *model;
 
@@ -563,8 +632,8 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 			if (gtk_widget_is_sensitive(dialog_data->ensure_directory) &&
 			    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->ensure_directory)))
 			{
-				if (!gtk_widget_is_sensitive(dialog_data->extract_full) ||
-				    !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->extract_full)) ||
+				if (!gtk_widget_is_sensitive(dialog_data->full_paths_radio) ||
+				    !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->full_paths_radio)) ||
 				    !xa_has_containing_directory(archive))
 				{
 					gchar *extraction_dir;
@@ -590,10 +659,22 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 				archive->do_overwrite = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->overwrite_check));
 			if (gtk_widget_is_sensitive(dialog_data->touch))
 				archive->do_touch = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->touch));
-			if (xa_main_window && gtk_widget_is_sensitive(dialog_data->extract_full))
-				archive->do_full_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->extract_full));
+
+			if (gtk_widget_is_sensitive(dialog_data->full_paths_radio) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->full_paths_radio)))
+			{
+				archive->do_full_path = TRUE;
+				strip = FALSE;
+			}
+			else if (gtk_widget_is_sensitive(dialog_data->relative_paths_radio) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->relative_paths_radio)))
+			{
+				archive->do_full_path = TRUE;
+				strip = TRUE;
+			}
 			else
-				archive->do_full_path = archive->can_full_path[0];
+			{
+				archive->do_full_path = FALSE;
+				strip = FALSE;
+			}
 
 			if (gtk_widget_is_sensitive(dialog_data->fresh))
 				archive->do_freshen = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog_data->fresh));
@@ -614,13 +695,9 @@ void xa_parse_extract_dialog_options (XArchive *archive,Extract_dialog_data *dia
 			else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (dialog_data->selected_radio)))
 				gtk_tree_selection_selected_foreach(selection,(GtkTreeSelectionForeachFunc)xa_concat_selected_filenames,&names);
 			if (xa_main_window)
-			{
 				xa_set_button_state(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, 0);
-				gtk_label_set_text(GTK_LABEL(total_label),_("Extracting files from archive, please wait..."));
-			}
 
-			archive->status = XARCHIVESTATUS_EXTRACT;
-			(*archive->archiver->extract)(archive,names);
+			xa_execute_extract_commands(archive, names, strip);
 		}
 	}
 	gtk_widget_hide (dialog_data->dialog1);
@@ -637,7 +714,7 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	GtkTreeSelection *selection;
 	GtkTreeViewColumn *column;
 	GSList *radiobutton1_group;
-	char *column_names[]= {(_("Archive Name")),(_("Size")),(_("Path")),NULL};
+	char *column_names[] = {_("Archive"), _("Size"), _("Path"), NULL};
 	int x;
 
 	dialog_data = g_new0 (Multi_extract_data,1);
@@ -646,13 +723,14 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_window_set_position (GTK_WINDOW (dialog_data->multi_extract),GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_type_hint (GTK_WINDOW (dialog_data->multi_extract),GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog_data->multi_extract),TRUE);
-	gtk_widget_set_size_request(dialog_data->multi_extract,-1,300);
+	gtk_widget_set_size_request(dialog_data->multi_extract, -1, 310);
 	gtk_window_set_title (GTK_WINDOW (dialog_data->multi_extract),_("Multi-Extract"));
 
 	xa_set_xarchiver_icon(GTK_WINDOW(dialog_data->multi_extract));
 
 	dialog_vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog_data->multi_extract));
-	vbox1 = gtk_vbox_new (FALSE,5);
+	vbox1 = gtk_vbox_new(FALSE, 6);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 2);
 	gtk_box_pack_start (GTK_BOX (dialog_vbox1),vbox1,TRUE,TRUE,0);
 	scrolledwindow1 = gtk_scrolled_window_new (NULL,NULL);
 	gtk_box_pack_start (GTK_BOX (vbox1),scrolledwindow1,TRUE,TRUE,0);
@@ -678,11 +756,14 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(dialog_data->files_treeview),TRUE);
 	gtk_container_add (GTK_CONTAINER (scrolledwindow1),dialog_data->files_treeview);
 
-	gtk_drag_dest_set(dialog_data->files_treeview, GTK_DEST_DEFAULT_ALL, drop_targets, 1, GDK_ACTION_COPY);
+	gtk_drag_dest_set(dialog_data->files_treeview, GTK_DEST_DEFAULT_ALL, drop_targets, G_N_ELEMENTS(drop_targets), GDK_ACTION_COPY);
 	g_signal_connect (G_OBJECT (dialog_data->files_treeview),"drag-data-received",G_CALLBACK (xa_multi_extract_dialog_drag_data_received),dialog_data);
 
-	hbox2 = gtk_hbox_new (TRUE,5);
-	gtk_box_pack_start (GTK_BOX (vbox1),hbox2,FALSE,TRUE,0);
+	hbox2 = gtk_hbox_new(TRUE, 0);
+	alignment1 = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_container_add(GTK_CONTAINER(alignment1), hbox2);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 4, 0, 0, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), alignment1, FALSE, FALSE, 0);
 
 	add_button = gtk_button_new_from_stock ("gtk-add");
 	gtk_box_pack_end (GTK_BOX (hbox2),add_button,FALSE,FALSE,0);
@@ -698,19 +779,20 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 
 	/* Destination dirs frame */
 	hbox1 = gtk_hbox_new (TRUE,8);
-	gtk_box_pack_start (GTK_BOX (vbox1),hbox1,FALSE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
 	frame1 = gtk_frame_new (NULL);
 	gtk_box_pack_start (GTK_BOX (hbox1),frame1,TRUE,TRUE,0);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame1),GTK_SHADOW_OUT);
 	alignment1 = gtk_alignment_new (0.5,0.5,1,1);
 	gtk_container_add (GTK_CONTAINER (frame1),alignment1);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment1),0,0,12,0);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment1), 0, 0, 12, 2);
 
 	vbox2 = gtk_vbox_new (TRUE,0);
 	gtk_container_add (GTK_CONTAINER (alignment1),vbox2);
-	hbox3 = gtk_hbox_new (FALSE,2);
+	hbox3 = gtk_hbox_new(FALSE, 4);
 	gtk_box_pack_start (GTK_BOX (vbox2),hbox3,FALSE,FALSE,0);
 	dialog_data->extract_to = gtk_radio_button_new_with_mnemonic (NULL,_("Extract to:"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->extract_to), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox3),dialog_data->extract_to,FALSE,FALSE,0);
 	radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dialog_data->extract_to));
 
@@ -719,10 +801,10 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_box_pack_start (GTK_BOX (hbox3),dialog_data->entry1,TRUE,TRUE,0);
 	gtk_entry_set_activates_default(GTK_ENTRY(dialog_data->entry1), TRUE);
 
-	dialog_data->extract_to_archive_name = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Extract to dir \"Archive Name\""));
-	gtk_widget_set_tooltip_text(dialog_data->extract_to_archive_name, _("This option extracts archives in directories named with the archive names"));
+	dialog_data->extract_to_archive_name = gtk_radio_button_new_with_mnemonic(radiobutton1_group, _("Extract to directories with archive names"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->extract_to_archive_name), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox2),dialog_data->extract_to_archive_name,FALSE,FALSE,0);
-	label1 = gtk_label_new (_("Destination dirs:"));
+	label1 = gtk_label_new(_("Destination"));
 	gtk_frame_set_label_widget (GTK_FRAME (frame1),label1);
 
 	/* Option frame */
@@ -735,11 +817,13 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	vbox3 = gtk_vbox_new (TRUE,0);
 	gtk_container_add (GTK_CONTAINER (alignment2),vbox3);
 	dialog_data->overwrite = gtk_check_button_new_with_mnemonic (_("Overwrite existing files"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->overwrite), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->overwrite,FALSE,FALSE,0);
-	dialog_data->full_path = gtk_check_button_new_with_mnemonic (_("Extract pathnames"));
+	dialog_data->full_path = gtk_check_button_new_with_mnemonic(_("Extract with full path"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(dialog_data->full_path), FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox3),dialog_data->full_path,FALSE,FALSE,0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog_data->full_path), TRUE);
-	label2 = gtk_label_new (_("Options:"));
+	label2 = gtk_label_new(_("Options"));
 	gtk_frame_set_label_widget(GTK_FRAME(frame2),label2);
 
 	cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
@@ -778,7 +862,13 @@ void xa_multi_extract_dialog_add_file (gchar *file_path, Multi_extract_data *dia
 
 	if (!archiver[xa.type].list)
 	{
-		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
+		gchar *name, *msg;
+
+		name = g_path_get_basename(file_path);
+		msg = g_strdup_printf(_("The proper archiver for \"%s\" is not installed!"), name);
+		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), msg);
+		g_free(msg);
+		g_free(name);
 		return;
 	}
 
@@ -887,3 +977,67 @@ run:
 	dialog->nr=0;
 	gtk_list_store_clear(dialog->files_liststore);
 }
+
+void xa_execute_extract_commands (XArchive *archive, GSList *list, gboolean strip)
+{
+	gchar *extract_to, *extraction_dir, *command;
+
+	if (xa_main_window)
+	{
+		/* normal extraction if current location isn't inside a directory */
+		if (archive->location_path == NULL)
+			strip = FALSE;
+	}
+	else
+	{
+		/* normal extraction if there is no containing directory */
+		if (!xa_has_containing_directory(archive))
+			strip = FALSE;
+		else
+			archive->location_path = archive->root_entry->child->filename;
+	}
+
+	if (strip)
+	{
+		if (!xa_create_working_directory(archive))
+		{
+			xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't create directory!"), "");
+			return;
+		}
+
+		extract_to = g_strconcat(archive->working_dir, "/xa-tmp.XXXXXX", NULL);
+
+		if (!g_mkdtemp(extract_to))
+		{
+			xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't create directory!"), "");
+			g_free(extract_to);
+			return;
+		}
+
+		extraction_dir = g_shell_quote(archive->extraction_dir);
+		g_free(archive->extraction_dir);
+		archive->extraction_dir = xa_escape_bad_chars(extract_to, ESCAPES);
+	}
+
+	archive->status = XARCHIVESTATUS_EXTRACT;
+	(*archive->archiver->extract)(archive, list);
+
+	if (strip)
+	{
+		archive->child_dir = g_strconcat(extract_to, "/", archive->location_path, NULL);
+
+		command = g_strconcat("sh -c \"exec mv",
+		                      archive->do_overwrite ? " -f" : (archive->do_update ? " -fu" : " -n"),
+		                      " -- `ls -A` ", extraction_dir, "\"", NULL);
+
+		archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
+		xa_run_command(archive, command);
+		g_free(command);
+
+		g_free(archive->child_dir);
+		archive->child_dir = NULL;
+
+		g_free(extraction_dir);
+		g_free(extract_to);
+	}
+}
