@@ -536,14 +536,20 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_LZIP;
 	path = g_find_program_in_path("lzip");
 
-	if (!path && plzip)
+	if (path)
+		archiver[type].is_compressor = TRUE;
+	else if (plzip)
+	{
 		/* alternative compressor */
 		path = g_strdup(plzip);
+		archiver[type].is_compressor = TRUE;
+	}
+	else
+		path = g_find_program_in_path("lunzip");
 
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
 		archiver[type].type = g_slist_append(archiver[type].type, "lzip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz");
 
@@ -979,22 +985,6 @@ static void xa_check_available_archivers ()
 	g_free(unar);
 }
 
-static gchar *xa_make_full_path (char *filename)
-{
-	gchar *cur_dir, *path;
-
-	if (*filename != '/')
-	{
-		cur_dir = g_get_current_dir();
-		path = g_strconcat(cur_dir, "/", filename, NULL);
-		g_free(cur_dir);
-	}
-	else
-		path = g_strdup(filename);
-
-	return path;
-}
-
 static XArchive *xa_init_structure_from_cmd_line (char *filename)
 {
 	XArchive *archive;
@@ -1008,7 +998,13 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 
 	if (!archiver[xa.type].list)
 	{
-		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
+		gchar *name, *msg;
+
+		name = g_path_get_basename(filename);
+		msg = g_strdup_printf(_("The proper archiver for \"%s\" is not installed!"), name);
+		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), msg);
+		g_free(msg);
+		g_free(name);
 		return NULL;
 	}
 
@@ -1155,6 +1151,7 @@ int main (int argc, char **argv)
 
 			archive->do_full_path = TRUE;
 			archive->do_overwrite = TRUE;
+			archive->do_touch = FALSE;
 			archive->extraction_dir = xa_escape_bad_chars(opt_extract_path, ESCAPES);
 			g_free(extraction_dir);
 
@@ -1195,11 +1192,20 @@ int main (int argc, char **argv)
 		/* Switch -m */
 		else if (opt_multi_extract)
 		{
-			Multi_extract_data *multi_extract = NULL;
+			Multi_extract_data *multi_extract;
+			gchar *arg;
+
 			multi_extract = xa_create_multi_extract_dialog();
+
 			for (i = 1; i < argc; i++)
-				if (!g_file_test(argv[i], G_FILE_TEST_IS_DIR))
-					xa_multi_extract_dialog_add_file(argv[i], multi_extract);
+			{
+				arg = xa_make_full_path(argv[i]);
+
+				if (!g_file_test(arg, G_FILE_TEST_IS_DIR))
+					xa_multi_extract_dialog_add_file(arg, multi_extract);
+
+				g_free(arg);
+			}
 
 			xa_multi_extract_dialog(multi_extract);
 			gtk_widget_destroy (multi_extract->multi_extract);
@@ -1227,6 +1233,7 @@ int main (int argc, char **argv)
 			{
 				gchar *fname;
 				gboolean is_dir;
+				GSList *files = NULL;
 
 				fname = xa_make_full_path(opt_compress);
 				is_dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
@@ -1240,21 +1247,26 @@ int main (int argc, char **argv)
 				}
 
 				xa_create_working_directory(archive);
-				current_dir = g_path_get_dirname(opt_compress);
-				chdir(current_dir);
-				g_free(current_dir);
-				GSList *files = NULL;
-				current_dir = g_path_get_basename(opt_compress);
-				files = g_slist_append(files, g_strdup(current_dir));
-				g_free(current_dir);
+
+				archive->child_dir = g_path_get_dirname(opt_compress);
+
+				fname = g_path_get_basename(opt_compress);
+				files = g_slist_append(files, fname);
 				g_free(opt_compress);
+
 				for (i = 1; i < argc; i++)
 				{
-					current_dir = g_path_get_basename(argv[i]);
-					files = g_slist_append(files, g_strdup(current_dir));
-					g_free(current_dir);
+					fname = g_path_get_basename(argv[i]);
+					files = g_slist_append(files, fname);
 				}
-				xa_execute_add_commands(archive, files, TRUE, NULL);
+
+				archive->do_full_path = FALSE;
+				archive->do_update = FALSE;
+				archive->do_freshen = FALSE;
+				archive->do_move = FALSE;
+				archive->do_solid = FALSE;
+
+				xa_execute_add_commands(archive, files, TRUE);
 			}
 		}
 		/* Switch -a */
