@@ -98,8 +98,9 @@ static void xa_prefs_dialog_set_default_options (Prefs_dialog_data *prefs_data)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->allow_sub_dir),FALSE);
 	/* Set the default options in the extract dialog */
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory), TRUE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (extract_window->extract_full),TRUE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->full_paths_radio), TRUE);
 	/* Set the default options in the add dialog */
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_window->update), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (add_window->recurse),TRUE);
 }
 
@@ -135,6 +136,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	GdkPixbuf *icon_pixbuf;
 	GtkTreePath *top;
 	Prefs_dialog_data *prefs_data;
+	GList *children;
 
 	prefs_data = g_new0 (Prefs_dialog_data,1);
 	prefs_data->dialog1 = gtk_dialog_new_with_buttons(_("Preferences"),
@@ -156,21 +158,25 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 
 	prefs_data->prefs_liststore = gtk_list_store_new (3,GDK_TYPE_PIXBUF,G_TYPE_STRING,G_TYPE_UINT);
 	gtk_list_store_append (prefs_data->prefs_liststore,&iter);
-	icon_pixbuf = gtk_icon_theme_load_icon(icon_theme, "gnome-mime-application-zip", 40, (GtkIconLookupFlags) 0, NULL);
-	if (!icon_pixbuf) icon_pixbuf = gtk_icon_theme_load_icon(icon_theme,"package-x-generic",32,GTK_ICON_LOOKUP_FORCE_SIZE,NULL);
+	icon_pixbuf = gtk_icon_theme_load_icon(icon_theme, "package-x-generic", 32, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 	gtk_list_store_set (prefs_data->prefs_liststore, &iter, 0, icon_pixbuf, 1, _("Archive"),2,0,-1);
-	if(icon_pixbuf != NULL)
-		g_object_unref (icon_pixbuf);
+
+	if (icon_pixbuf)
+		g_object_unref(icon_pixbuf);
 
 	gtk_list_store_append (prefs_data->prefs_liststore, &iter);
-	icon_pixbuf = gtk_widget_render_icon (prefs_data->dialog1, "gtk-leave-fullscreen", GTK_ICON_SIZE_DND, NULL);
+	icon_pixbuf = gtk_icon_theme_load_icon(icon_theme, "view-fullscreen", 32, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 	gtk_list_store_set (prefs_data->prefs_liststore, &iter, 0, icon_pixbuf, 1, _("Window"),2,1,-1);
-	g_object_unref (icon_pixbuf);
+
+	if (icon_pixbuf)
+		g_object_unref(icon_pixbuf);
 
 	gtk_list_store_append (prefs_data->prefs_liststore, &iter);
-	icon_pixbuf = gtk_widget_render_icon (prefs_data->dialog1, "gtk-execute", GTK_ICON_SIZE_DND, NULL);
+	icon_pixbuf = gtk_icon_theme_load_icon(icon_theme, "system-run", 32, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 	gtk_list_store_set (prefs_data->prefs_liststore, &iter, 0, icon_pixbuf, 1, _("Advanced"),2,2,-1);
-	g_object_unref (icon_pixbuf);
+
+	if (icon_pixbuf)
+		g_object_unref(icon_pixbuf);
 
 	prefs_data->iconview = gtk_icon_view_new_with_model(GTK_TREE_MODEL(prefs_data->prefs_liststore));
 	g_object_unref (prefs_data->prefs_liststore);
@@ -201,6 +207,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE,0);
 
 	prefs_data->combo_prefered_format = gtk_combo_box_text_new();
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_format), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox1), prefs_data->combo_prefered_format,FALSE,TRUE,0);
 
 	prefs_data->prefer_unzip = gtk_check_button_new_with_mnemonic(_("Prefer unzip for zip files (requires restart)"));
@@ -235,6 +242,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	                  GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);
 	prefs_data->combo_icon_size = gtk_combo_box_text_new();
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_icon_size), FALSE);
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_icon_size), _("small"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_icon_size), _("small/medium"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_icon_size), _("medium"));
@@ -285,6 +293,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 		                  GTK_FILL, GTK_SHRINK, 0, 0);
 		gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);
 		prefs_data->combo_prefered_web_browser = gtk_combo_box_text_new();
+		gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_web_browser), FALSE);
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_web_browser), "");
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_web_browser), _("choose..."));
 		g_signal_connect (prefs_data->combo_prefered_web_browser,"changed",G_CALLBACK (xa_prefs_combo_changed),NULL);
@@ -296,6 +305,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 		                  GTK_FILL, GTK_SHRINK, 0, 0);
 		gtk_misc_set_alignment (GTK_MISC (label7), 0, 0.5);
 		prefs_data->combo_prefered_editor = gtk_combo_box_text_new();
+		gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_editor), FALSE);
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_editor), "");
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_editor), _("choose..."));
 		g_signal_connect (prefs_data->combo_prefered_editor,"changed",G_CALLBACK (xa_prefs_combo_changed),NULL);
@@ -307,6 +317,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 		                  GTK_FILL, GTK_SHRINK, 0, 0);
 		gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);
 		prefs_data->combo_prefered_viewer = gtk_combo_box_text_new();
+		gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_viewer), FALSE);
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_viewer), "");
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_viewer), _("choose..."));
 		g_signal_connect (prefs_data->combo_prefered_viewer,"changed",G_CALLBACK (xa_prefs_combo_changed),NULL);
@@ -318,6 +329,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 		                 GTK_FILL, GTK_SHRINK, 0, 0);
 		gtk_misc_set_alignment(GTK_MISC(label8), 0, 0.5);
 		prefs_data->combo_prefered_archiver = gtk_combo_box_text_new();
+		gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_archiver), FALSE);
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), "");
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), _("choose..."));
 		g_signal_connect(prefs_data->combo_prefered_archiver, "changed", G_CALLBACK(xa_prefs_combo_changed), NULL);
@@ -330,6 +342,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	                 GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label8), 0, 0.5);
 	prefs_data->combo_prefered_custom_cmd = gtk_combo_box_text_new();
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_custom_cmd), FALSE);
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), "");
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), _("choose..."));
 	g_signal_connect(prefs_data->combo_prefered_custom_cmd, "changed", G_CALLBACK(xa_prefs_combo_changed), NULL);
@@ -341,6 +354,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	                 GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);
 	prefs_data->combo_prefered_temp_dir = gtk_combo_box_text_new();
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_temp_dir), FALSE);
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_temp_dir), _("/tmp"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_temp_dir), _("choose..."));
 	g_signal_connect (prefs_data->combo_prefered_temp_dir,"changed",G_CALLBACK (xa_prefs_combo_changed),GUINT_TO_POINTER(1));
@@ -352,6 +366,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	                 GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);
 	prefs_data->combo_prefered_extract_dir = gtk_combo_box_text_new();
+	gtk_combo_box_set_focus_on_click(GTK_COMBO_BOX(prefs_data->combo_prefered_extract_dir), FALSE);
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_extract_dir), _("/tmp"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_extract_dir), _("choose..."));
 	g_signal_connect (prefs_data->combo_prefered_extract_dir,"changed",G_CALLBACK (xa_prefs_combo_changed),GUINT_TO_POINTER(1));
@@ -362,16 +377,21 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	gtk_table_attach(GTK_TABLE(table2), prefs_data->check_save_geometry, 0, 2, 7, 8,
 	                 GTK_FILL, GTK_FILL,0, 0);
 
-	prefs_data->allow_sub_dir = gtk_check_button_new_with_mnemonic (_("Allow subdirs with drag and drop"));
+	prefs_data->allow_sub_dir = gtk_check_button_new_with_mnemonic (_("Allow subdirs with clipboard and drag-and-drop"));
+	gtk_button_set_focus_on_click(GTK_BUTTON(prefs_data->allow_sub_dir), FALSE);
+	children = gtk_container_get_children(GTK_CONTAINER(prefs_data->allow_sub_dir));
+	gtk_label_set_line_wrap(GTK_LABEL(children->data), TRUE);
+	g_list_free(children);
 	gtk_table_attach(GTK_TABLE(table2), prefs_data->allow_sub_dir, 0, 2, 8, 9,
 	                 GTK_FILL, (GtkAttachOptions) 0, 0, 0);
-	gtk_widget_set_tooltip_text(prefs_data->allow_sub_dir, _("This option includes the subdirectories when you add files with drag and drop"));
+	gtk_widget_set_tooltip_text(prefs_data->allow_sub_dir, _("This option includes the subdirectories when you add files from the clipboard or with drag-and-drop"));
 	gtk_button_set_focus_on_click (GTK_BUTTON (prefs_data->check_save_geometry), FALSE);
 
 	if (!xdg_open)
 	{
-		label5 = gtk_label_new(_("<span color='red' style='italic'>Please install xdg-utils package so that\nXarchiver can recognize more file types.</span>"));
+		label5 = gtk_label_new(_("\n<span color='red' style='italic'>Please install xdg-utils package so that\nXarchiver can recognize more file types.</span>"));
 		gtk_label_set_use_markup (GTK_LABEL (label5), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(label5), 0, 0);
 		gtk_box_pack_start (GTK_BOX (vbox3), label5, FALSE, FALSE, 0);
 	}
 	label3 = gtk_label_new ("");
@@ -384,6 +404,7 @@ void xa_prefs_save_options(Prefs_dialog_data *prefs_data, const char *filename)
 	gchar *conf;
 	gchar *value= NULL;
 	gsize len;
+	gint val;
 
 	if (access(filename, F_OK) == 0 && access(filename, W_OK) == -1)
 		return;
@@ -470,7 +491,16 @@ void xa_prefs_save_options(Prefs_dialog_data *prefs_data, const char *filename)
 	/* Save the options in the extract dialog */
 	g_key_file_set_boolean(xa_key_file, PACKAGE, "ensure_directory", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"overwrite",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->overwrite_check)));
-	g_key_file_set_boolean (xa_key_file,PACKAGE,"full_path",gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->extract_full)));
+
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extract_window->full_paths_radio)))
+		val = 2;
+	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(extract_window->relative_paths_radio)))
+		val = 1;
+	else
+		val = 0;
+
+	g_key_file_set_integer(xa_key_file, PACKAGE, "full_path", val);
+
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"touch",    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->touch)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"fresh",  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->fresh)));
 	g_key_file_set_boolean (xa_key_file,PACKAGE,"update",   gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (extract_window->update)));
@@ -500,7 +530,7 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 	GKeyFile *xa_key_file = g_key_file_new();
 	GError *error = NULL;
 	gboolean unzip, toolbar;
-	gint idx;
+	gint idx, val;
 
 	config_dir = g_strconcat (g_get_home_dir(),"/.config",NULL);
 	if (g_file_test(config_dir, G_FILE_TEST_EXISTS) == FALSE)
@@ -676,7 +706,24 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 		/* Load the options in the extract dialog */
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->ensure_directory), g_key_file_get_boolean(xa_key_file, PACKAGE, "ensure_directory", NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->overwrite_check),g_key_file_get_boolean(xa_key_file,PACKAGE,"overwrite",NULL));
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->extract_full),g_key_file_get_boolean(xa_key_file,PACKAGE,"full_path",NULL));
+
+		val = g_key_file_get_integer(xa_key_file, PACKAGE, "full_path", &error);
+
+		if (error)
+		{
+			/* set default setting with existing, old config file */
+			val = 2;
+			g_error_free(error);
+			error = NULL;
+		}
+
+		if (val == 2)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->full_paths_radio), TRUE);
+		else if (val == 1)
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->relative_paths_radio), TRUE);
+		else
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(extract_window->no_paths_radio), TRUE);
+
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->touch),g_key_file_get_boolean(xa_key_file,PACKAGE,"touch",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->fresh),g_key_file_get_boolean(xa_key_file,PACKAGE,"fresh",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(extract_window->update),g_key_file_get_boolean(xa_key_file,PACKAGE,"update",NULL));
@@ -684,6 +731,11 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->store_path),g_key_file_get_boolean(xa_key_file,PACKAGE,"store_path",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->update),g_key_file_get_boolean(xa_key_file,PACKAGE,"updadd",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->freshen),g_key_file_get_boolean(xa_key_file,PACKAGE,"freshen",NULL));
+
+		/* set default setting with existing, old config file */
+		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_window->update)) && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_window->freshen)))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(add_window->overwrite), TRUE);
+
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->recurse),g_key_file_get_boolean(xa_key_file,PACKAGE,"recurse",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->solid_archive),g_key_file_get_boolean(xa_key_file,PACKAGE,"solid_archive",NULL));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(add_window->remove_files),g_key_file_get_boolean(xa_key_file,PACKAGE,"remove_files",NULL));
