@@ -140,6 +140,10 @@ static gchar *xa_rpm2cpio (XArchive *archive)
 			executable = "xz -dc ";
 			break;
 
+		case XARCHIVETYPE_ZSTD:
+			executable = "zstd -dc ";
+			break;
+
 		default:
 			g_free(cpio_z);
 			return g_strdup(_("Unknown compression type!"));
@@ -148,6 +152,7 @@ static gchar *xa_rpm2cpio (XArchive *archive)
 	command = g_strconcat("sh -c \"exec ", executable, cpio_z, " > ", archive->working_dir, "/xa-tmp.cpio\"", NULL);
 	g_free(cpio_z);
 
+	archive->status = XARCHIVESTATUS_LIST;        // restore status
 	success = xa_run_command(archive, command);
 	g_free(command);
 
@@ -329,7 +334,7 @@ gboolean xa_rpm_extract (XArchive *archive, GSList *file_list)
 	                      archive->do_touch ? "" : " -m",
 	                      archive->do_overwrite ? " -u" : "",
 	                      " -I ", archive->working_dir, "/xa-tmp.cpio",
-	                      files->str, NULL);
+	                      " --", files->str, NULL);
 	result = xa_run_command(archive, command);
 	g_free(command);
 
@@ -344,9 +349,10 @@ gboolean xa_rpm_extract (XArchive *archive, GSList *file_list)
 		archive->child_dir = g_strdup(extract_to);
 		command = g_strconcat("mv",
 		                      archive->do_overwrite ? " -f" : " -n",
-		                      all_files->str, " ", archive->extraction_dir, NULL);
+		                      " --", all_files->str, " ", archive->extraction_dir, NULL);
 		g_string_free(all_files, TRUE);
 
+		archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
 		result = xa_run_command(archive, command);
 		g_free(command);
 
