@@ -169,7 +169,7 @@ static gchar *xa_strip_current_working_dir_from_path (gchar *working_dir, gchar
 	return filename+len;
 }
 
-static void xa_remove_slash_from_path (gchar *path)
+void xa_remove_slash_from_path (gchar *path)
 {
 	size_t len = strlen(path);
 
@@ -408,7 +408,8 @@ GString *xa_collect_files_in_dir (const gchar *directory)
 		file = stack->data;
 		stack = g_slist_delete_link(stack, stack);
 
-		if (g_file_test(file, G_FILE_TEST_IS_DIR))
+		if (g_file_test(file, G_FILE_TEST_IS_DIR) &&
+		   !g_file_test(file, G_FILE_TEST_IS_SYMLINK))
 		{
 			GDir *dir;
 			const gchar *name;
@@ -489,3 +490,19 @@ gchar *xa_set_max_width_chars_ellipsize (const gchar *string, gint n, PangoEllip
 
 	return ellipsized;
 }
+
+gchar *xa_make_full_path (const char *filename)
+{
+	gchar *cur_dir, *path;
+
+	if (*filename != '/')
+	{
+		cur_dir = g_get_current_dir();
+		path = g_strconcat(cur_dir, "/", filename, NULL);
+		g_free(cur_dir);
+	}
+	else
+		path = g_strdup(filename);
+
+	return path;
+}
