@@ -269,7 +269,7 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 	                      " -f ", archive->path[2],
 	                      archive->do_touch ? " -m" : "",
 	                      archive->do_overwrite ? "" : (archive->do_update ? " --keep-newer-files" : " -k"),
-	                      " -C ", extract_to, files->str, NULL);
+	                      " -C ", extract_to, " --", files->str, NULL);
 
 	result = xa_run_command(archive, command);
 	g_free(command);
@@ -281,11 +281,11 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 
 		archive->child_dir = g_strdup(extract_to);
 		command = g_strconcat("mv",
-		                      archive->do_overwrite ? " -f" : " -n",
-		                      archive->do_update ? " -fu" : "",
-		                      all_files->str, " ", archive->extraction_dir, NULL);
+		                      archive->do_overwrite ? " -f" : (archive->do_update ? " -fu" : " -n"),
+		                      " --", all_files->str, " ", archive->extraction_dir, NULL);
 		g_string_free(all_files, TRUE);
 
+		archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
 		result = xa_run_command(archive, command);
 		g_free(command);
 
@@ -299,13 +299,11 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 	return result;
 }
 
-void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
+void xa_tar_add (XArchive *archive, GSList *file_list)
 {
 	GString *files;
 	gchar *command;
-
-	if (archive->location_path != NULL)
-		archive->child_dir = g_strdup(archive->working_dir);
+	gboolean success;
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 
@@ -331,27 +329,33 @@ void xa_tar_add (XArchive *archive, GSList *file_list, gchar *compression)
 		command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0],
 		                      " -c --no-recursion --no-wildcards",
 		                      archive->do_move ? " --remove-files" : "",
-		                      " -f ", archive->path[2], files->str, NULL);
+		                      " -f ", archive->path[2], " --", files->str, NULL);
 	}
 	else
 		command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0],
 		                      archive->do_update ? " -u" : " -r",
 		                      " --no-recursion --no-wildcards",
 		                      archive->do_move ? " --remove-files" : "",
-		                      " -f ", archive->path[2], files->str, NULL);
+		                      " -f ", archive->path[2], " --", files->str, NULL);
 
 	if (archive->type != XARCHIVETYPE_TAR)
 	{
-		xa_run_command(archive, command);
+		success = xa_run_command(archive, command);
 		g_free(command);
+		command = NULL;
 
-		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
-		archive->status = XARCHIVESTATUS_ADD;
+		if (success)
+		{
+			command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
+			archive->status = XARCHIVESTATUS_ADD;   // restore status
+		}
 	}
 
 	g_string_free(files, TRUE);
 
-	xa_run_command(archive, command);
+	if (command)
+		xa_run_command(archive, command);
+
 	g_free(command);
 }
 
@@ -359,21 +363,28 @@ void xa_tar_delete (XArchive *archive, GSList *file_list)
 {
 	GString *files;
 	gchar *command;
+	gboolean success;
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
-	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0], " --delete --no-recursion --no-wildcards -f ", archive->path[2], files->str, NULL);
+	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0], " --delete --no-recursion --no-wildcards -f ", archive->path[2], " --", files->str, NULL);
 
 	if (archive->type != XARCHIVETYPE_TAR)
 	{
-		xa_run_command(archive, command);
+		success = xa_run_command(archive, command);
 		g_free(command);
+		command = NULL;
 
-		command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
-		archive->status = XARCHIVESTATUS_DELETE;
+		if (success)
+		{
+			command = xa_gzip_et_al_get_command(archiver[xa_tar_get_compressor_type(archive)].program[0], archive->path[2], archive->path[0], archive->password, xa_tar_get_compressor_type(archive));
+			archive->status = XARCHIVESTATUS_DELETE;   // restore status
+		}
 	}
 
 	g_string_free(files, TRUE);
 
-	xa_run_command(archive, command);
+	if (command)
+		xa_run_command(archive, command);
+
 	g_free(command);
 }
