@@ -235,10 +235,11 @@ gboolean xa_unar_extract (XArchive *archive, GSList *file_list)
 
 	files = xa_quote_filenames(file_list, "*?[]\"", TRUE);
 	password_str = xa_unar_password_str(archive);
-	command = g_strconcat(archiver[archive->type].program[1], " ",
-	                      archive->path[1], password_str, " -D -q",
+	command = g_strconcat(archiver[archive->type].program[1],
+	                      password_str, " -D -q",
 	                      archive->do_overwrite ? " -f" : "",
-	                      " -o ", extract_to, files->str, NULL);
+	                      " -o ", extract_to,
+	                      " ", archive->path[1], " --", files->str, NULL);
 
 	result = xa_run_command(archive, command);
 	g_free(command);
@@ -251,9 +252,10 @@ gboolean xa_unar_extract (XArchive *archive, GSList *file_list)
 		archive->child_dir = g_strdup(extract_to);
 		command = g_strconcat("mv",
 		                      archive->do_overwrite ? " -f" : " -n",
-		                      all_files->str, " ", archive->extraction_dir, NULL);
+		                      " --", all_files->str, " ", archive->extraction_dir, NULL);
 		g_string_free(all_files, TRUE);
 
+		archive->status = XARCHIVESTATUS_EXTRACT;   // restore status
 		result = xa_run_command(archive, command);
 		g_free(command);
 
