@@ -28,6 +28,7 @@
 void xa_zip_ask (XArchive *archive)
 {
 	gchar *sfx;
+	compressor_t zip_compressor = {TRUE, 1, 6, 9, 1};
 
 	sfx = g_find_program_in_path("unzipsfx");
 
@@ -46,6 +47,9 @@ void xa_zip_ask (XArchive *archive)
 	archive->can_freshen[0] = TRUE;
 	archive->can_freshen[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
+	archive->can_compress = archiver[archive->type].is_compressor;
+	archive->compressor = zip_compressor;
+	archive->compression = archive->compressor.preset;
 
 	g_free(sfx);
 }
@@ -256,63 +260,62 @@ gboolean xa_zip_extract (XArchive *archive, GSList *file_list)
 	return result;
 }
 
-void xa_zip_add (XArchive *archive, GSList *file_list, gchar *compression)
+void xa_zip_add (XArchive *archive, GSList *file_list)
 {
 	gboolean epub;
 	GString *files;
-	gchar *password_str, *command;
+	gchar *compression, *password_str, *command;
 
-	if (archive->location_path != NULL)
-		archive->child_dir = g_strdup(archive->working_dir);
-
-	if (!compression)
-		compression = "6";
+	compression = g_strdup_printf("%hu", archive->compression);
 
 	epub = (archive->tag == 'e');
 
-	/* epub requires mimetype to be uncompressed, unencrypted, and the first file in the archive*/
+	/* epub requires mimetype to be uncompressed, unencrypted, and the first file in the archive */
 	if (epub)
 	{
-		guint i;
-		GSList *fname;
+		GSList *flist, *fname = NULL;
 
-		for (i = 0; i < g_slist_length(file_list); i++)
+		flist = file_list;
+
+		while (flist)
 		{
-			fname = g_slist_nth(file_list, i);
-
-			if (strcmp(fname->data, "mimetype") == 0)
+			if (strcmp(flist->data, "mimetype") == 0)
+			{
+				fname = flist;
 				break;
+			}
 
-			fname = NULL;
+			flist = flist->next;
 		}
 
 		if (fname)
 		{
 			command = g_strconcat(archiver[archive->type].program[1], " -q",
-			                      archive->do_update ? " -ou" : "",
-			                      archive->do_freshen ? " -of" : "",
+			                      archive->do_update ? " -ou" : (archive->do_freshen ? " -of" : ""),
 			                      archive->do_move ? " -m" : "",
 			                      " -0X ",
 			                      archive->path[1], " mimetype", NULL);
 			xa_run_command(archive, command);
 			g_free(command);
 
-			file_list = g_slist_remove(file_list, g_slist_nth_data(file_list, i));
+			archive->status = XARCHIVESTATUS_ADD;   // preserve status
+
+			file_list = g_slist_remove(file_list, fname);
 		}
 	}
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);   // no escaping for adding!
 	password_str = xa_zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[1], " -q",
-	                      archive->do_update ? " -ou" : "",
-	                      archive->do_freshen ? " -of" : "",
+	                      archive->do_update ? " -ou" : (archive->do_freshen ? " -of" : ""),
 	                      archive->do_move ? " -m" : "",
 	                      " -", compression,
 	                      epub ? "XD" : "",
 	                      password_str, " ",
-	                      archive->path[1], files->str, NULL);
+	                      archive->path[1], " --", files->str, NULL);
 	g_free(password_str);
 	g_string_free(files,TRUE);
+	g_free(compression);
 
 	xa_run_command(archive, command);
 	g_free(command);
@@ -324,7 +327,7 @@ void xa_zip_delete (XArchive *archive, GSList *file_list)
 	gchar *command;
 
 	files = xa_quote_filenames(file_list, "*?[]", TRUE);
-	command = g_strconcat(archiver[archive->type].program[1], " -d ", archive->path[1], files->str, NULL);
+	command = g_strconcat(archiver[archive->type].program[1], " -d ", archive->path[1], " --", files->str, NULL);
 	g_string_free(files,TRUE);
 
 	xa_run_command(archive, command);
